/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.ws.notification.handlers.request;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPBody;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPFactory;
import org.apache.axiom.soap.SOAPHeader;
import org.objectweb.petals.ws.fault.WsnFault;
import org.objectweb.petals.ws.notification.Subscribe;
import org.objectweb.petals.ws.notification.SubscribeResponse;
import org.objectweb.petals.ws.notification.Subscription;
import org.objectweb.petals.ws.notification.WsnConstants;
import org.objectweb.petals.ws.notification.WsnHelper;

/**
 * @author chamerling - eBM WebSourcing
 *
 */
public class SubscribeHandler extends WSNHandler {

    /**
     * Creates a new instance of {@link SubscribeHandler}
     *
     * @param topics
     */
    public SubscribeHandler() {
        super(WsnConstants.SUBSCRIBE_URI, WsnConstants.SUBSCRIBE_QNAME);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.binding.soap.wsn.handlers.WSNHandler#handle(org.apache.axiom.soap.SOAPEnvelope)
     */
    public SOAPEnvelope handle(SOAPEnvelope envelope) throws WsnFault {
        SOAPEnvelope reply = null;
        SOAPBody body = envelope.getBody();

        try {

            // build the subscribe object
            OMElement subscribeElement = body
                    .getFirstChildWithName(WsnConstants.SUBSCRIBE_QNAME);
            Subscribe subscribe = WsnHelper.subscribeFromOM(subscribeElement);

            Subscription subscription = new Subscription();
            subscription.setProducerEPR(this.manager.getProducerEPR());
            subscription.setConsumerEPR(subscribe.getConsumerEPR());
            subscription.setFilter(subscribe.getFilter());

            this.manager.addSubscription(subscription);

            reply = buildSubscriptionReply(subscribe);

        } catch (Exception e) {
            reply = buildErrorReply();
        }
        return reply;
    }

    /**
     * Build the subscription reply
     *
     * @param sub
     * @return
     */
    protected SOAPEnvelope buildSubscriptionReply(Subscribe sub)
            throws Exception {
        SOAPEnvelope reply = null;
        SOAPFactory factory = OMAbstractFactory.getSOAP11Factory();
        reply = factory.createSOAPEnvelope();

        SOAPBody body = factory.createSOAPBody();

        // subscription reference
        SubscribeResponse response = new SubscribeResponse();
        response.setSubscriptionEPR(sub.getConsumerEPR());

        OMElement subscribeResponseElement = WsnHelper
                .subscribeResponseToOM(response);
        body.addChild(subscribeResponseElement);

        // create SOAP header
        SOAPHeader header = factory.createSOAPHeader();

        // add SOAP header
        reply.addChild(header);

        // add SOAP body
        reply.addChild(body);
        return reply;
    }

    /**
     * Build the error soap message
     *
     * @param sub
     * @return
     */
    protected SOAPEnvelope buildErrorReply() {
        SOAPEnvelope reply = null;
        SOAPFactory factory = OMAbstractFactory.getSOAP11Factory();
        reply = factory.createSOAPEnvelope();
        // TODO
        return reply;
    }
}
