/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.ws.notification.handlers.request;

import java.net.URI;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPBody;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPHeader;
import org.objectweb.petals.ws.addressing.EndpointReference;
import org.objectweb.petals.ws.addressing.WsaConstants;
import org.objectweb.petals.ws.fault.WsnFault;
import org.objectweb.petals.ws.notification.Unsubscribe;
import org.objectweb.petals.ws.notification.UnsubscribeResponse;
import org.objectweb.petals.ws.notification.WsnConstants;
import org.objectweb.petals.ws.notification.WsnHelper;
import org.objectweb.petals.ws.util.AxiomHelper;

/**
 * @author Christophe HAMERLING (chamerling) - eBM WebSourcing
 *
 */
public class UnsubscribeHandler extends WSNHandler {

    /**
     * Creates a new instance of {@link UnsubscribeHandler}
     *
     */
    public UnsubscribeHandler() {
        super(WsnConstants.UNSUBSCRIBE_URI, WsnConstants.UNSUBSCRIBE_QNAME);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.ws.notification.handlers.request.WSNHandler#handle(org.apache.axiom.soap.SOAPEnvelope)
     */
    @Override
    public SOAPEnvelope handle(SOAPEnvelope envelope) throws WsnFault {
        SOAPEnvelope response = null;
        SOAPBody body = envelope.getBody();
        SOAPHeader header = envelope.getHeader();

        try {

            // build the subscribe object
            OMElement unsubElement = header
                    .getFirstChildWithName(WsaConstants.FROM_QNAME);
            EndpointReference epr = new EndpointReference(URI
                    .create(unsubElement.getText()));

            Unsubscribe unsubscribe = WsnHelper.unsubscribeFromSOAPBody(body);
            unsubscribe.setConsumerEPR(epr);
            unsubscribe.setProducerEPR(this.manager.getProducerEPR());

            this.manager.removeSubscription(epr);

            response = buildUnsubscribeReply(unsubscribe);

        } catch (Exception e) {
            response = buildErrorReply();
        }

        return response;
    }

    /**
     *
     * @return
     */
    protected SOAPEnvelope buildErrorReply() {
        // TODO
        return AxiomHelper.createSOAPEnvelope();
    }

    /**
     *
     * @param unsubscribe
     * @return
     */
    protected SOAPEnvelope buildUnsubscribeReply(Unsubscribe unsubscribe) {
        SOAPEnvelope envelope = AxiomHelper.createSOAPEnvelope();
        SOAPBody body = AxiomHelper.createSOAPBody(envelope);
        SOAPHeader header = AxiomHelper.createSOAPHeader(envelope);

        // add action element
        OMElement actionElement = AxiomHelper
                .createOMElement(WsaConstants.ACTION_QNAME);
        actionElement.setText(WsnConstants.UNSUBSCRIBE_RESPONSE_URI);
        header.addChild(actionElement);

        // TODO : add EPRs

        UnsubscribeResponse response = new UnsubscribeResponse();
        response.setConsumerEPR(unsubscribe.getConsumerEPR());
        response.setProducerEPR(unsubscribe.getProducerEPR());
        // TODO : get content

        // add unsubscribeElement
        OMElement responseElement = AxiomHelper
                .createOMElement(WsnConstants.UNSUBSCRIBE_RESPONSE_QNAME);
        body.addChild(responseElement);

        return envelope;
    }
}
